var async = require( 'async' ),
    Set = require( 'simplesets' ).Set;

var PER_PAGE = 3;

function collect ( items, key ) {
  var itemIds = new Set();
  for ( var i = 0, l = items.length; i < l; i++ ) {
    for ( var e = 0, k = items[ i ][ key ].length; e < k; e++ ) {
      itemIds.add( items[ i ][ key ][ e ] );
    }
  }
  return itemIds.array();
}

function authors ( ramen, posts, cb ) {
  var authorIds = collect( posts, 'authors' );
  ramen.authors( authorIds ).run( cb );
}

function tags ( ramen, posts, cb ) {
  var tagIds = collect( posts, 'tags' );
  ramen.tags( tagIds ).run( cb );
}

module.exports = {

  home: function ( ramen, cb ) {
    this.page( 1, ramen, cb );
  },

  page: function ( page, ramen, cb ) {
    page = Number( page );
    ramen
      // get posts
      .posts()
      // start on the appropriate page
      .start( ( page - 1 ) * PER_PAGE )
      // limit the number returned
      .limit( PER_PAGE )
      // sort by publish date
      .sort( 'pubdate', 'desc' )
      // only return posts published before now
      .filter( 'pubdate', 'lt', Date.now() )

      .run( function ( err, data ) {
        if ( err ) {
          cb( err );
        } else {
          var response_data = {
            posts: data,
            page: page
          };

          async.parallel({
            authors: function ( cb ) { authors( ramen, response_data.posts, cb ); },
            tags: function ( cb ) { tags( ramen, response_data.posts, cb ); }
          }, function ( err, data ) {
            if ( err ) {
              cb( err );
            } else {
              response_data.authors = data.authors;
              response_data.tags = data.tags;
              cb( null, response_data );
            }
          });
        }
      });
  },

  post: function ( ymd, url, ramen, cb ) {
    ramen
      // get posts
      .posts()
      // limit to one post
      .limit( 1 )
      // filter by date and url
      .filter( 'pubdate', 'on', ymd )
      .filter( 'url', 'eq', url )

      .run( function ( err, data ) {
        if ( err ) {
          cb( err );
        } else if ( data.length !== 1 ) {
          cb( { code: 404, type: 'client', description: 'Not found' } );
        } else {
          var post = data[ 0 ];

          async.parallel({
            authors: function ( cb ) { authors( ramen, [ post ], cb ); },
            tags: function ( cb ) { tags( ramen, [ post ], cb ); }
          }, function ( err, data ) {
            if ( err ) {
              cb( err );
            } else {
              data.post = post;
              data.page_title = post.title;
              cb( null, data );
            }
          });
        }
      });
  },

  author: function ( url, ramen, cb ) {
    ramen
      // get authors
      .authors()
      // limit to one author
      .limit( 1 )
      // filter by url
      .filter( 'url', 'eq', url )

      .run( function ( err, data ) {
        if ( err ) {
          cb( err );
        } else if ( data.length !== 1 ) {
          cb( { code: 404, type: 'client', description: 'Not found' } );
        } else {
          ramen
            // get posts
            .posts()
            // only posts who's author list contains this author
            .filter( 'authors', 'eq', data[ 0 ]._id )
            // only published posts
            .filter( 'pubdate', 'lte', Date.now() )
            // sort by pubdate
            .sort( 'pubdate', 'desc' )

            .run ( function ( err, data2 ) {
              if ( err ) {
                cb( err );
              } else {
                cb( null, {
                  author: data[ 0 ],
                  posts: data2,
                  page_title: data[ 0 ].name
                });
              }
            });
        }
      });
  },

  tag: function ( url, ramen, cb ) {
    ramen
      // get tags 
      .tags()
      // limit to one tag
      .limit( 1 )
      // filter by url
      .filter( 'url', 'eq', url )

      .run( function ( err, data ) {
        if ( err ) {
          cb( err );
        } else if ( data.length !== 1 ) {
          cb( { code: 404, type: 'client', description: 'Not found' } );
        } else {
          ramen
            // get posts
            .posts()
            // only posts who's tag list contains this tag
            .filter( 'tags', 'eq', data[ 0 ]._id )
            // only published posts
            .filter( 'pubdate', 'lte', Date.now() )
            // sort by pubdate
            .sort( 'pubdate', 'desc' )

            .run ( function ( err, data2 ) {
              if ( err ) {
                cb( err );
              } else {
                cb( null, {
                  tag: data[ 0 ],
                  posts: data2,
                  page_title: data[ 0 ].title
                });
              }
            });
        }
      });
  },

  archive: function ( ymd, ramen, cb ) {
    ramen
      .posts()
      .filter( 'pubdate', 'on', ymd )
      .sort( 'pubdate', 'desc' )

      .run( function ( err, data ) {
        if ( err ) {
          cb( err );
        } else {
          cb( null, { ymd: ymd, posts: data } );
        }
      });
  },

  rss: function ( ramen, cb ) {
    ramen
      .posts()
      .filter( 'pubdate', 'lte', Date.now() )
      .sort( 'pubdate', 'desc' )

      .run( function ( err, data ) {
        if ( err ) {
          cb( err );
        } else {
          cb( null, data );
        }
      });
  }
};

