var express = require( 'express' ),
    jqtpl = require( 'jqtpl' ),
    RSS = require( 'rss' ),
    data_methods = require( './data-methods' ),
    tpl_methods = require( './tpl-methods' ),
    Ramen = require( 'ramen-api' );

module.exports = {
  createServer: createServer
};

function merge ( a, b ) {
  function C () {}
  Object.keys( a ).forEach( function ( key ) {
    C.prototype[ key ] = a[ key ];
  });
  Object.keys( b ).forEach( function ( key ) {
    C.prototype[ key ] = b[ key ];
  });
  return new C();
}

function createServer ( options ) {
  var Data = merge( data_methods, options.data_methods || {} ),
      Tpl = merge( tpl_methods, options.tpl_methods || {} );

  function fetch_and_render ( res, type, args ) {
    function done ( err, data ) {
      if ( err ) {
        res.render( 'error', { error: err } );
      } else {
        data.tpl = Tpl;
        if ( !data.page_title ) {
          data.page_title = options.default_title;
        }
        res.render( type, data );
      }
    }

    args = args || [];

    args.push( ramen );
    args.push( done );

    Data[ type ].apply( Data, args );
  }

  var ramen = new Ramen({
    url: options.ramen_url,
    blog: options.ramen_blog,
    user: options.ramen_user,
    password: options.ramen_password
  });

  var app = express.createServer();

  app.configure( function () {
    app.set( 'views', options.views );
    app.set( 'view engine', 'html' );
    app.register( '.html', require( 'jqtpl' ).express );

    app.use( express.logger() );
    app.use( app.router );
    //app.use( express.errorHandler() );
  });

  app.get( '/', function ( req, res ) {
    fetch_and_render( res, 'home' );
  });

  // /YYYY-MM-DD/post-url
  app.get( /^\/(\d{4})\-(\d{2})\-(\d{2})\/(.+)$/, function ( req, res ) {
    var ymd = req.params.slice( 0, 3 ),
        url = req.params[ 3 ];

    fetch_and_render( res, 'post', [ ymd, url ] );
  });

  app.get( '/page/:page', function ( req, res ) {
    fetch_and_render( res, 'page', [ req.param( 'page' ) ] );
  });

  app.get( '/authors/:url', function ( req, res ) {
    fetch_and_render( res, 'author', [ req.param( 'url' ) ] );
  });

  app.get( '/tags/:url', function ( req, res ) {
    fetch_and_render( res, 'tag', [ req.param( 'url' ) ] );
  });

  app.get( /^\/(\d{4})$/, function ( req, res ) {
    fetch_and_render( res, 'archive', [ req.params ] );
  });

  app.get( /^\/(\d{4})\-(\d{2})$/, function ( req, res ) {
    fetch_and_render( res, 'archive', [ req.params ] );
  });

  app.get( /^\/(\d{4})\-(\d{2})\-(\d{2})$/, function ( req, res ) {
    fetch_and_render( res, 'archive', [ req.params ] );
  });

  app.get( '/feed.xml', function ( req, res ) {
    Data.rss( ramen, function ( err, posts ) {
      if ( err ) {
        res.send( 500 );
      } else {
        var feed = new RSS({
          title: options.rss_title,
          description: options.rss_description,
          feed_url: options.site_url + '/feed.xml',
          site_url: options.site_url,
          author: options.rss_author
        });

        posts.forEach( function ( post ) {
          feed.item({
            title: post.title,
            url: options.site_url + tpl.post_url( post ),
            date: post.pubdate.toLocaleString()
          });
        });

        res.header( 'Content-type', 'application/rss+xml' );
        res.send( feed.xml() );
      }
    });
  });

  return {
    app: app,
    start: function ( cb ) {
      app.listen( options.port, cb );
    },
    stop: function () {
      app.close();
    }
  };
}

